package com.xwkj.jcjcgz.mvp.ui.fragment;

import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Message;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.github.mikephil.charting.charts.PieChart;
import com.github.mikephil.charting.data.PieData;
import com.github.mikephil.charting.data.PieDataSet;
import com.github.mikephil.charting.data.PieEntry;
import com.jess.arms.base.BaseFragment;
import com.jess.arms.di.component.AppComponent;
import com.jess.arms.utils.ArmsUtils;

import com.xwkj.jcjcgz.di.component.DaggerRwfxTaskTypeComponent;
import com.xwkj.jcjcgz.mvp.contract.RwfxTaskTypeContract;
import com.xwkj.jcjcgz.mvp.presenter.RwfxTaskTypePresenter;

import com.xwkj.jcjcgz.R;
import com.xwkj.jcjcgz.mvp.ui.adapter.RwfxTaskTypeAdapter;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;

import static com.jess.arms.utils.Preconditions.checkNotNull;


/**
 * ================================================
 * Description:
 * <p>
 * Created by MVPArmsTemplate on 07/12/2024 10:07
 * <a href="mailto:jess.yan.effort@gmail.com">Contact me</a>
 * <a href="https://github.com/JessYanCoding">Follow me</a>
 * <a href="https://github.com/JessYanCoding/MVPArms">Star me</a>
 * <a href="https://github.com/JessYanCoding/MVPArms/wiki">See me</a>
 * <a href="https://github.com/JessYanCoding/MVPArmsTemplate">模版请保持更新</a>
 * ================================================
 */
public class RwfxTaskTypeFragment extends BaseFragment<RwfxTaskTypePresenter> implements RwfxTaskTypeContract.View {

    @BindView(R.id.rwfx_task_type_rv)
    RecyclerView recyclerView;
    @BindView(R.id.rwfx_task_type_chart)
    PieChart chart;

    public static RwfxTaskTypeFragment newInstance() {
        RwfxTaskTypeFragment fragment = new RwfxTaskTypeFragment();
        return fragment;
    }

    @Override
    public void setupFragmentComponent(@NonNull AppComponent appComponent) {
        DaggerRwfxTaskTypeComponent //如找不到该类,请编译一下项目
                .builder()
                .appComponent(appComponent)
                .view(this)
                .build()
                .inject(this);
    }

    @Override
    public View initView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_rwfx_task_type, container, false);
    }

    @Override
    public void initData(@Nullable Bundle savedInstanceState) {
        recyclerView.setNestedScrollingEnabled(false);
        recyclerView.setFocusable(false);
        recyclerView.setLayoutManager(new LinearLayoutManager(getActivity()));
        List<String> list = new ArrayList<>();
        list.add("1");
        list.add("1");
        list.add("1");
        RwfxTaskTypeAdapter adapter = new RwfxTaskTypeAdapter(list);
        recyclerView.setAdapter(adapter);
        init();
    }
    private void init() {
        List<PieEntry> list = new ArrayList<>();
        list.add(new PieEntry(2f,"米饭"));
        list.add(new PieEntry(3f,"鸡肉"));
        list.add(new PieEntry(5f,"鱼肉"));
        PieDataSet pieDataSet=new PieDataSet(list,"吃的饭菜");
        //为每个分组设置颜色，如果不为每个都设置颜色最后饼图只会有一个颜色，这样无法有效看出占比
        pieDataSet.setColors(Color.GREEN,Color.BLUE,Color.RED);
        PieData pieData=new PieData(pieDataSet);
        chart.setData(pieData);


        //设置图形上文字的大小
        pieDataSet.setValueTextSize(16f);
        //设置图形上数值的颜色
        pieDataSet.setValueTextColor(Color.YELLOW);
        //是否绘制饼图上的文字
        // pieDataSet.setDrawValues(true);
        //文字绘制在图形内还是图形外(这里绘制在外部。默认在内部)
        pieDataSet.setXValuePosition(PieDataSet.ValuePosition.OUTSIDE_SLICE);
        //图形上的数值绘制在图形内还是图形外(这里绘制在外部。默认在内部)
        pieDataSet.setYValuePosition(PieDataSet.ValuePosition.OUTSIDE_SLICE);
        //连线的颜色
        pieDataSet.setValueLineColor(getResources().getColor(R.color.colorAccent));
        // 设置图形上文字的颜色
        chart.setEntryLabelColor(getResources().getColor(R.color.colorAccent));
        //设置连线长度
        pieDataSet.setValueLinePart1Length(1f);
        //设置连线长度是否可变
        pieDataSet.setValueLineVariableLength(false);
        //设置连接线距离饼图的距离
        pieDataSet.setValueLinePart1OffsetPercentage(100f);
    }
    /**
     * 通过此方法可以使 Fragment 能够与外界做一些交互和通信, 比如说外部的 Activity 想让自己持有的某个 Fragment 对象执行一些方法,
     * 建议在有多个需要与外界交互的方法时, 统一传 {@link Message}, 通过 what 字段来区分不同的方法, 在 {@link #setData(Object)}
     * 方法中就可以 {@code switch} 做不同的操作, 这样就可以用统一的入口方法做多个不同的操作, 可以起到分发的作用
     * <p>
     * 调用此方法时请注意调用时 Fragment 的生命周期, 如果调用 {@link #setData(Object)} 方法时 {@link Fragment#onCreate(Bundle)} 还没执行
     * 但在 {@link #setData(Object)} 里却调用了 Presenter 的方法, 是会报空的, 因为 Dagger 注入是在 {@link Fragment#onCreate(Bundle)} 方法中执行的
     * 然后才创建的 Presenter, 如果要做一些初始化操作,可以不必让外部调用 {@link #setData(Object)}, 在 {@link #initData(Bundle)} 中初始化就可以了
     * <p>
     * Example usage:
     * <pre>
     * public void setData(@Nullable Object data) {
     *     if (data != null && data instanceof Message) {
     *         switch (((Message) data).what) {
     *             case 0:
     *                 loadData(((Message) data).arg1);
     *                 break;
     *             case 1:
     *                 refreshUI();
     *                 break;
     *             default:
     *                 //do something
     *                 break;
     *         }
     *     }
     * }
     *
     * // call setData(Object):
     * Message data = new Message();
     * data.what = 0;
     * data.arg1 = 1;
     * fragment.setData(data);
     * </pre>
     *
     * @param data 当不需要参数时 {@code data} 可以为 {@code null}
     */
    @Override
    public void setData(@Nullable Object data) {

    }

    @Override
    public void showLoading() {

    }

    @Override
    public void hideLoading() {

    }

    @Override
    public void showMessage(@NonNull String message) {
        checkNotNull(message);
        ArmsUtils.snackbarText(message);
    }

    @Override
    public void launchActivity(@NonNull Intent intent) {
        checkNotNull(intent);
        ArmsUtils.startActivity(intent);
    }

    @Override
    public void killMyself() {

    }
}
